diff roundup/backends/back_postgresql.py @ 6610:db3f0ba75b4a

Change checkpoint_data and restore_connection_on_error to subtransaction checkpoint_data and restore_connection_on_error used to commit() and rollback() the db connection. This causes additional I/O and load. Changed them to use 'SAVEPOINT name' and 'ROLLBACK TO name' to get a faster method for handling errors within a tranaction. One thing to note is that postgresql (unlike SQL std) doesn't overwrite an older savepoint with he same name. It keeps all savepoints but only rolls back to the newest one with a given name. This could be a resource issue. I left a commented out release statement in case somebody runs into an issue due to too many savepoints. I expect it to slow down the import but....
author John Rouillard <rouilj@ieee.org>
date Sat, 29 Jan 2022 11:29:36 -0500
parents 0d99ae7c8de6
children bdd28b244839
line wrap: on
line diff
--- a/roundup/backends/back_postgresql.py	Fri Jan 28 20:11:32 2022 -0500
+++ b/roundup/backends/back_postgresql.py	Sat Jan 29 11:29:36 2022 -0500
@@ -203,21 +203,33 @@
             self._add_fts_table()
             self.commit()
 
-    def checkpoint_data(self):
-        """Commit the state of the database. Allows recovery/retry
-           of operation in exception handler because postgres
-           requires a rollback in case of error generating exception
+    def checkpoint_data(self, savepoint="importing"):
+        """Create a subtransaction savepoint. Allows recovery/retry
+           of operation in exception handler because
+           postgres requires a rollback in case of error
+           generating exception.  Used with
+           restore_connecion_on_error to handle uniqueness
+           conflict in import_table().
         """
-        self.commit()
+        # Savepoints take resources. Postgres keeps all
+        # savepoints (rather than overwriting) until a
+        # commit(). If an import fails because of a resource
+        # issue with savepoints, uncomment this line. I
+        # expect it will slow down the import but it should
+        # eliminate any issue with stored savepoints and
+        # resource use.
+        #
+        # self.sql('RELEASE SAVEPOINT %s' % savepoint)
+        self.sql('SAVEPOINT %s' % savepoint)
 
-    def restore_connection_on_error(self):
-        """Postgres leaves a cursor in an unusable state after
-           an error. Rollback the transaction to recover and
-           permit a retry of the failed statement. Used with
-           checkpoint_data to handle uniqueness conflict in
-           import_table()
+    def restore_connection_on_error(self, savepoint="importing"):
+        """Postgres leaves a connection/cursor in an unusable state
+           after an error. Rollback the transaction to a
+           previous savepoint and permit a retry of the
+           failed statement. Used with checkpoint_data to
+           handle uniqueness conflict in import_table().
         """
-        self.rollback()
+        self.sql('ROLLBACK TO %s' % savepoint)
 
     def create_version_2_tables(self):
         # OTK store

Roundup Issue Tracker: http://roundup-tracker.org/